#include "zlib/zlib.h"

#if !defined(PACKED)
#	if defined(__GNUC__)
#		define PACKED __attribute__((packed))
#	else
#		define PACKED
#	endif
#endif

/* Custom defines */
#define charmax(_var) sizeof(_var)-1
#define _exit   system("pause");\
                return 0;
    
#define console_print printf
#define formatex snprintf

#define MAX_PLUGIN_STRING 250
        
/* Types */
typedef uint32_t		dword;
typedef uint64_t		qword;
typedef uint16_t		word;
typedef uint8_t			byte;

typedef int32_t			cell;
typedef uint32_t		ucell;

/* STRUCTs */
typedef struct tagAMX_HEADER 
{
  dword 	size          PACKED; /* size of the "file" */
  word 		magic         PACKED; /* signature */
  byte		file_version;         /* file format version */
  byte		amx_version;          /* required version of the AMX */
  word		flags         PACKED;
  word		defsize       PACKED; /* size of a definition record */
  dword		cod           PACKED; /* initial value of COD - code block */
  dword		dat           PACKED; /* initial value of DAT - data block */
  dword		hea           PACKED; /* initial value of HEA - start of the heap */
  dword		stp           PACKED; /* initial value of STP - stack top */
  dword		cip           PACKED; /* initial value of CIP - the instruction pointer */
  dword		publics       PACKED; /* offset to the "public functions" table */
  dword		natives       PACKED; /* offset to the "native functions" table */
  dword		libraries     PACKED; /* offset to the table of libraries */
  dword		pubvars       PACKED; /* the "public variables" table */
  dword		tags          PACKED; /* the "public tagnames" table */
  dword		nametable     PACKED; /* name table */
} PACKED AMX_HEADER;

typedef struct plugin_s
{
	dword 			 magic;				/* magic number */
	word	 		 version;			/* magic version */
	byte 			 sections;			/* how many plugins are in this package */
	byte 			 cellsize;			/* size of cells for this plugin, in bytes. */
	dword	 		 disksize;			/* compressed file size */
	dword 			 imagesize;			/* uncompressed file size */
	dword 			 memsize;			/* memory image size */
	dword	 		 offs;				/* file offset */
	dword			 headersize;		/* The size of this file header. */
	
	void			*data;				/* the uncompressed data after the initial header is read. */
	AMX_HEADER		*amx_header;
} plugin_t;
			
plugin_t plugin;

/* Funtions */
bool read_header(FILE *fp);
void write_header(FILE *fp);
void load_uncompress(FILE *fp);
void dump_header_data();
bool amxx_uncompress(FILE *fp);
bool amxx_compress(FILE *fp);
bool get_amxx_header();
int generate_strings();
int load_memory_file();

/*File Read */
void read_dword(FILE *fp, dword *ret)
{
	fread(ret,1,sizeof(dword),fp);
}

void read_word(FILE *fp, word *ret)
{
	fread(ret,1,sizeof(word),fp);
}

void read_byte(FILE *fp, byte *ret)
{
	fread(ret,1,sizeof(byte),fp);
}

/*File Write */
void write_dword(FILE *fp, dword *num)
{
	fwrite(num, sizeof(dword), 1, fp);
}

void write_word(FILE *fp, word *num)
{
	fwrite(num, sizeof(word), 1, fp);
}

void write_byte(FILE *fp, byte *num)
{
	fwrite(num, sizeof(byte), 1, fp);
}
